//
//  JJAnimationController.m
//  SideslipProject
//
//  Created by 姜克勤 on 2017/7/18.
//  Copyright © 2017年 jkq. All rights reserved.
//

#import "JJAnimationController.h"
#import "JJSliderMeunTool.h"

@interface JJAnimationController ()

@property (strong, nonatomic) UIViewController *rootController;
@property (strong, nonatomic) UIViewController *leftController;
@property (strong, nonatomic) UIView *backView;

@property (assign, nonatomic) BOOL hasShow;

@end

static NSTimeInterval animationTime = 0.2;


@implementation JJAnimationController


-(instancetype)initWithRootViewController:(UIViewController *)rootViewController contentViewController:(UIViewController *)leftViewController{
    
    self = [super init];
    
    if (self) {
        _rootController = rootViewController;
        _leftController = leftViewController;
    }
    
    return self;
}

/** 控制状态栏 */
- (BOOL)prefersStatusBarHidden{
    return self.hideStatusBar;
}

-(UIStatusBarStyle)preferredStatusBarStyle{
    return UIStatusBarStyleLightContent;
}

-(UIStatusBarAnimation)preferredStatusBarUpdateAnimation{
    return UIStatusBarAnimationSlide;
}


- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.view.backgroundColor = [UIColor clearColor];
    //半透明
    CGRect frame = [UIScreen mainScreen].bounds;
    self.backView = [[UIView alloc] initWithFrame:frame];
    self.backView.backgroundColor = [UIColor blackColor];
    self.backView.alpha = 0.0;
    [self.view addSubview:self.backView];
    
    /** 两个手势 */
    /* 点击 */
    UITapGestureRecognizer *tapGestureRec = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(closeSideBar)];
    [self.backView addGestureRecognizer:tapGestureRec];
    /* 滑动 */
    UIPanGestureRecognizer *panGestureRec = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(moveViewWithGesture:)];
    [self.view addGestureRecognizer:panGestureRec];
    
    self.leftController.view.backgroundColor = [UIColor redColor];
    CGFloat width = [UIScreen mainScreen].bounds.size.width - 50;
    if ([UIScreen mainScreen].bounds.size.width > 375) {
        width -= 50;
    } else if ([UIScreen mainScreen].bounds.size.width > 320) {
        width = width - 25;
    }
    self.leftController.view.frame = CGRectMake(-width, 0, width, [UIScreen mainScreen].bounds.size.height);
    self.leftController.view.backgroundColor = [UIColor groupTableViewBackgroundColor];
    [self.view addSubview:self.leftController.view];
    [self addChildViewController:self.leftController];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    __weak JJAnimationController *weakSelf = self;
    
    if (!self.hasShow) {
        self.hasShow = YES;
        self.hideStatusBar = YES;
        [UIView animateWithDuration:animationTime animations:^{
            [weakSelf setNeedsStatusBarAppearanceUpdate];
            weakSelf.rootController.navigationController.navigationBar.frame = CGRectMake(0, 0, [UIScreen mainScreen].bounds.size.width, 64);
        }];
    }
    [self.navigationController setNavigationBarHidden:YES animated:YES];
    [self showAnimation];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [self.navigationController setNavigationBarHidden:NO animated:YES];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/**
 * 点击手势
 */
- (void)closeSideBar {
    [self closeAnimation];
}

/**
 * 拖拽手势
 */
- (void)moveViewWithGesture:(UIPanGestureRecognizer *)panGes {
    // 下面是计算
    // 开始位置
    static CGFloat startX;
    // 结束位置
    static CGFloat lastX;
    // 改变多少
    static CGFloat durationX;
    CGPoint touchPoint = [panGes locationInView:[[UIApplication sharedApplication] keyWindow]];
    
    CGFloat leftController_width = self.leftController.view.frame.size.width;
    CGFloat leftController_height = self.leftController.view.frame.size.height;
    CGFloat leftController_x = self.leftController.view.frame.origin.x;
    
    // 手势开始
    if (panGes.state == UIGestureRecognizerStateBegan) {
        startX = touchPoint.x;
        lastX = touchPoint.x;
    }
    // 手势改变
    if (panGes.state == UIGestureRecognizerStateChanged) {
        CGFloat currentX = touchPoint.x;
        durationX = currentX - lastX;
        lastX = currentX;
        CGFloat leftController_originx = durationX + leftController_x;
        // 如果控制器的x小于宽度直接返回
        if (leftController_originx <= -leftController_width) {
            leftController_originx = -leftController_width;
        }
        // 如果控制器的x大于0直接返回
        if (leftController_originx >= 0) {
            leftController_originx = 0;
        }
        // 计算bgView的透明度
        self.backView.alpha = (1 + leftController_originx / leftController_width) * 0.5;
        // 设置左边控制器的frame
        [self.leftController.view setFrame:CGRectMake(leftController_originx, 0, leftController_width, leftController_height)];
    }
    // 手势结束
    if (panGes.state == UIGestureRecognizerStateEnded) {
        // 结束为止超时屏幕一半
        if (leftController_x > - leftController_width + [UIScreen mainScreen].bounds.size.width / 2) {
            [self showAnimation];
        } else {
            [self closeAnimation];
        }
    }
}


/**
 显示控制器
 */
- (void)showAnimation {
    self.view.userInteractionEnabled = NO;
    //根据当前x，计算显示时间
    CGFloat time = fabs(self.leftController.view.frame.origin.x / self.leftController.view.frame.size.width) * animationTime;
    __weak JJAnimationController *weakSelf = self;
    [UIView animateWithDuration:time animations:^{
        weakSelf.leftController.view.frame = CGRectMake(0, 0, weakSelf.leftController.view.frame.size.width, [UIScreen mainScreen].bounds.size.height);
        weakSelf.backView.alpha = 0.3;
    } completion:^(BOOL finished) {
        weakSelf.view.userInteractionEnabled = YES;
    }];
}

- (void)closeAnimation {
    self.view.userInteractionEnabled = NO;
    CGFloat time = (1 - fabs(self.leftController.view.frame.origin.x / self.leftController.view.frame.size.width))*animationTime;
    __weak JJAnimationController *weakSelf = self;
    [UIView animateWithDuration:time animations:^{
        CGRect frame = weakSelf.leftController.view.frame;
        frame.origin.x = -frame.size.width;
        weakSelf.leftController.view.frame = frame;
    } completion:^(BOOL finished) {
        //状态栏出现
        weakSelf.hideStatusBar = NO;
        if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 10.0) {
            [UIView animateWithDuration:animationTime animations:^{
                [weakSelf setNeedsStatusBarAppearanceUpdate];
            }];
        }
        [JJSliderMeunTool dismissSlider];
    }];
}


/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
